@@ -60,7 +60,7 @@ func TestParseDir(t *testing.T) {
 func TestParseDirErrors(t *testing.T) {
 	// Confirm error cases: nonexistent dir; non-dir file; dir with *.sql files
 	// creating same table multiple times
-	for _, dirPath := range []string{"../bestdata", "../testdata/setup.sql", "../testdata"} {
+	for _, dirPath := range []string{"../../bestdata", "../../testdata/setup.sql", "../../testdata"} {
 		dir, err := ParseDir(dirPath, getValidConfig(t))
 		if err == nil || (dir != nil && dir.ParseError == nil) {
 			t.Errorf("Expected ParseDir to return nil dir and non-nil error, but dir=%v err=%v", dir, err)
@@ -71,10 +71,10 @@ func TestParseDirErrors(t *testing.T) {
 	cmd := mybase.NewCommand("fstest", "", "", nil)
 	cmd.AddArg("environment", "production", false)
 	cfg := mybase.ParseFakeCLI(t, cmd, "fstest")
-	if _, err := ParseDir("../testdata/golden/init/mydb", cfg); err == nil {
+	if _, err := ParseDir("../../testdata/golden/init/mydb", cfg); err == nil {
 		t.Error("Expected error from ParseDir(), but instead err is nil")
 	}
-	if _, err := ParseDir("../testdata/golden/init/mydb/product", cfg); err == nil {
+	if _, err := ParseDir("../../testdata/golden/init/mydb/product", cfg); err == nil {
 		t.Error("Expected error from ParseDir(), but instead err is nil")
 	}
 }
@@ -266,6 +266,87 @@ func TestParseDirNoSchemas(t *testing.T) {
 	}
 }
 
+func TestParseDirUnterminated(t *testing.T) {
+	// These 3 dirs have unterminated quotes, identifiers (backticks), and multi-
+	// line comments, respectively. Each should surface as dir.ParseError.
+	for _, subdir := range []string{"unterminatedcomment", "unterminatedident", "unterminatedquote"} {
+		if _, err := ParseDir("testdata/"+subdir, getValidConfig(t)); err == nil {
+			t.Errorf("In dir testdata/%s, expected error from ParseDir(), but instead err is nil", subdir)
+		}
+	}
+}
+
+func TestParseDirInvalidChar(t *testing.T) {
+	// This dir has an invalid operator ("?") which causes the lexer to fail.
+	// It should be handled as a fatal error and the position of the error
+	// should be correct.
+	_, err := ParseDir("testdata/invalidchar", getValidConfig(t))
+	if err == nil {
+		t.Fatal("In dir testdata/invalidchar, expected error from ParseDir(), but instead err is nil")
+	}
+	if msg := err.Error(); !strings.Contains(msg, "testdata/invalidchar/tables.sql:5:15: invalid token '?'") {
+		t.Errorf("Error message did not match expectation; result was %q", msg)
+	}
+}
+
+func TestParseDirUnknownIgnored(t *testing.T) {
+	// This dir contains an INSERT statement among the valid CREATEs. This should
+	// be tracked in dir.IgnoredStatements but isn't a fatal error.
+	if dir, err := ParseDir("testdata/unknownstatement", getValidConfig(t)); err != nil {
+		t.Fatalf("In dir testdata/unknownstatement, unexpected error from ParseDir(): %v", err)
+	} else if len(dir.LogicalSchemas) != 1 {
+		t.Fatalf("In dir testdata/unknownstatement, expected 1 logical schema, instead found %d", len(dir.LogicalSchemas))
+	} else if dir.ParseError != nil {
+		t.Fatalf("In dir testdata/unknownstatement, expected nil ParseError, instead found %v", dir.ParseError)
+	} else if len(dir.IgnoredStatements) != 1 {
+		t.Errorf("In dir testdata/unknownstatement, expected 1 IgnoredStatements, instead found %d", len(dir.IgnoredStatements))
+	}
+}
+
+func TestParseDirRedundantDelimiter(t *testing.T) {
+	// This dir contains two special cases of DELIMITER commands:
+	// * Setting a delimiter that is already the current delimiter, e.g. from ; to ;
+	// * Setting a delimiter that is double the previous delimiter, e.g. from ; to ;;
+	// These cases should not cause errors or IgnoredStatements.
+	if dir, err := ParseDir("testdata/redundantdelimiter", getValidConfig(t)); err != nil {
+		t.Fatalf("In dir testdata/redundantdelimiter, unexpected error from ParseDir(): %v", err)
+	} else if len(dir.LogicalSchemas) != 1 {
+		t.Fatalf("In dir testdata/redundantdelimiter, expected 1 logical schema, instead found %d", len(dir.LogicalSchemas))
+	} else if len(dir.LogicalSchemas[0].Creates) != 3 {
+		t.Fatalf("In dir testdata/redundantdelimiter, expected 3 CREATEs, instead found %d", len(dir.LogicalSchemas[0].Creates))
+	} else if dir.ParseError != nil {
+		t.Fatalf("In dir testdata/redundantdelimiter, expected nil ParseError, instead found %v", dir.ParseError)
+	} else if len(dir.IgnoredStatements) > 0 {
+		t.Errorf("In dir testdata/redundantdelimiter, expected 0 IgnoredStatements, instead found %d, first is %+v", len(dir.IgnoredStatements), *dir.IgnoredStatements[0])
+	}
+}
+
+func TestParseDirCreateSelect(t *testing.T) {
+	// This dir contains a CREATE ... SELECT statement, which is explicitly not
+	// supported at this time.
+	_, err := ParseDir("testdata/createselect", getValidConfig(t))
+	if err == nil {
+		t.Fatal("In dir testdata/createselect, expected error from ParseDir(), but instead err is nil")
+	}
+}
+
+func TestParseDirBOM(t *testing.T) {
+	// The .skeema file and tables.sql file in this dir both have a UTF8 byte-order
+	// marker prefix char, which should not interfere with the ability to parse the
+	// dir or its contents
+	if dir, err := ParseDir("testdata/utf8bom", getValidConfig(t)); err != nil {
+		t.Fatalf("In dir testdata/utf8bom, unexpected error from ParseDir(): %v", err)
+	} else if len(dir.LogicalSchemas) != 1 {
+		t.Fatalf("In dir testdata/utf8bom, expected 1 logical schema, instead found %d", len(dir.LogicalSchemas))
+	} else if len(dir.LogicalSchemas[0].Creates) != 2 {
+		t.Fatalf("In dir testdata/utf8bom, expected 2 CREATEs, instead found %d", len(dir.LogicalSchemas[0].Creates))
+	} else if dir.ParseError != nil {
+		t.Fatalf("In dir testdata/utf8bom, expected nil ParseError, instead found %v", dir.ParseError)
+	} else if len(dir.IgnoredStatements) != 0 {
+		t.Errorf("In dir testdata/utf8bom, expected 0 IgnoredStatements, instead found %d", len(dir.IgnoredStatements))
+	}
+}
+
 func TestDirBaseName(t *testing.T) {
 	dir := getDir(t, "../../testdata/golden/init/mydb/product")
 	if bn := dir.BaseName(); bn != "product" {
@@ -519,30 +600,33 @@ func TestAncestorPaths(t *testing.T) {
 	}
 }
 
-func getValidConfig(t *testing.T) *mybase.Config {
+func getValidConfigWithCLI(t *testing.T, cliOptions string) *mybase.Config {
+	t.Helper()
 	cmd := mybase.NewCommand("fstest", "", "", nil)
-	cmd.AddOption(mybase.StringOption("user", 'u', "root", "Username to connect to database host"))
-	cmd.AddOption(mybase.StringOption("password", 'p', "", "Password for database user; omit value to prompt from TTY (default no password)").ValueOptional())
-	cmd.AddOption(mybase.StringOption("schema", 0, "", "Database schema name").Hidden())
-	cmd.AddOption(mybase.StringOption("default-character-set", 0, "", "Schema-level default character set").Hidden())
-	cmd.AddOption(mybase.StringOption("default-collation", 0, "", "Schema-level default collation").Hidden())
-	cmd.AddOption(mybase.StringOption("host", 0, "", "Database hostname or IP address").Hidden())
-	cmd.AddOption(mybase.StringOption("port", 0, "3306", "Port to use for database host").Hidden())
-	cmd.AddOption(mybase.StringOption("flavor", 0, "", "Database server expressed in format vendor:major.minor, for use in vendor/version specific syntax").Hidden())
-	cmd.AddOption(mybase.StringOption("generator", 0, "", "Version of Skeema used for `skeema init` or most recent `skeema pull`").Hidden())
+	util.AddGlobalOptions(cmd)
 	cmd.AddArg("environment", "production", false)
-	return mybase.ParseFakeCLI(t, cmd, "fstest")
+	return mybase.ParseFakeCLI(t, cmd, "fstest "+cliOptions)
 }
 
-func getDir(t *testing.T, dirPath string) *Dir {
+func getDirWithCLI(t *testing.T, dirPath, cliOptions string) *Dir {
 	t.Helper()
-	dir, err := ParseDir(dirPath, getValidConfig(t))
+	dir, err := ParseDir(dirPath, getValidConfigWithCLI(t, cliOptions))
 	if err != nil {
 		t.Fatalf("Unexpected error parsing dir %s: %s", dirPath, err)
 	}
 	return dir
 }
 
+func getValidConfig(t *testing.T) *mybase.Config {
+	t.Helper()
+	return getValidConfigWithCLI(t, "")
+}
+
+func getDir(t *testing.T, dirPath string) *Dir {
+	t.Helper()
+	return getDirWithCLI(t, dirPath, "")
+}
+
 func countParseErrors(subs []*Dir) (count int) {
 	for _, sub := range subs {
 		if sub.ParseError != nil {
