@@ -3,10 +3,10 @@ package fs
 import (
 	"bufio"
 	"bytes"
+	"errors"
 	"fmt"
 	"io"
 	"os"
-	"regexp"
 	"strings"
 	"unicode"
 	"unicode/utf8"
@@ -27,7 +27,9 @@ const (
 	StatementTypeNoop                  // entirely whitespace and/or comments
 	StatementTypeCommand               // currently just USE or DELIMITER
 	StatementTypeCreate
-	StatementTypeAlter // not actually ever parsed yet
+	StatementTypeAlter     // not actually ever parsed yet
+	StatementTypeLexError  // something went horribly wrong, caller should treat as fatal
+	StatementTypeForbidden // disallowed statement such as CREATE TABLE ... SELECT
 	// Other types will be added once they are supported by the package
 )
 
@@ -45,7 +47,9 @@ type Statement struct {
 	ObjectName      string
 	ObjectQualifier string
 	FromFile        *TokenizedSQLFile
+	Error           error // any problem lexing or parsing this statement, populated when Type is StatementTypeUnknown, StatementTypeLexError, or StatementTypeForbidden
 	delimiter       string
+	nameClause      string // raw version, potentially with schema name qualifier and/or surrounding backticks; also any trailing whitespace/comments
 }
 
 // Location returns the file, line number, and character number where the
@@ -81,17 +85,20 @@ func (stmt *Statement) Schema() string {
 // whitespace, or qualified schema name.
 func (stmt *Statement) Body() string {
 	body, _ := stmt.SplitTextBody()
-	if stmt.ObjectQualifier != "" {
-		pattern := fmt.Sprintf("(?is)^(.*?create.+?%s.*?)(`%s`|%s)([\\s\\p{Zs}]*\\.[\\s\\p{Zs}]*)", string(stmt.ObjectType), stmt.ObjectQualifier, stmt.ObjectQualifier)
-		re := regexp.MustCompile(pattern)
-		body = re.ReplaceAllString(body, "$1")
+	if stmt.ObjectQualifier == "" || stmt.nameClause == "" {
+		return body
 	}
-	return body
+	// stmt.nameClause includes trailing whitespace, but we want to leave that intact
+	replaceClause := strings.TrimRight(stmt.nameClause, "\n\r\t ")
+	return strings.Replace(body, replaceClause, tengo.EscapeIdentifier(stmt.ObjectName), 1)
 }
 
 // SplitTextBody returns Text with its trailing delimiter and whitespace (if
 // any) separated out into a separate string.
 func (stmt *Statement) SplitTextBody() (body string, suffix string) {
+	if stmt == nil {
+		return "", ""
+	}
 	body = strings.TrimRight(stmt.Text, "\n\r\t ")
 	body = strings.TrimSuffix(body, stmt.delimiter)
 	body = strings.TrimRight(body, "\n\r\t ")
@@ -198,6 +205,12 @@ func (st *statementTokenizer) statements() ([]*Statement, error) {
 
 func (st *statementTokenizer) processLine(line string, eof bool) {
 	st.lineNo++
+
+	// Trim UTF8 BOM prefix if present at beginning of file
+	if st.lineNo == 1 {
+		line = strings.TrimPrefix(line, "\uFEFF")
+	}
+
 	ls := &lineState{
 		statementTokenizer: st,
 		line:               line,
@@ -257,16 +270,28 @@ func (st *statementTokenizer) processLine(line string, eof bool) {
 			ls.inRelevant = true
 		}
 
+		// Commands are special-cases in terms of delimiter vs newline handling: USE
+		// command has optional delimiter, and DELIMITER command has no delimiter
+		// (and requires special care to handle transititions like e.g. going from
+		// a single semicolon delimiter to double-semicolon delimiter!)
+		if bufLen := ls.buf.Len(); (bufLen == 4 || bufLen == 10) && unicode.IsSpace(c) { // potentially USE or DELIMITER followed by whitespace
+			var slurpLineAsCommand bool
+			if lowerBufStr := strings.ToLower(ls.buf.String()); strings.HasPrefix(lowerBufStr, "use") && !strings.Contains(ls.line[ls.pos:], st.delimiter) {
+				slurpLineAsCommand = true
+			} else if strings.HasPrefix(lowerBufStr, "delimiter") {
+				slurpLineAsCommand = true
+			}
+			if slurpLineAsCommand {
+				ls.buf.WriteString(ls.line[ls.pos:])
+				ls.stmt.delimiter = "\000" // prevent SplitTextBody from stripping previous delimiter from command arg
+				ls.doneStatement(0)
+				return
+			}
+		}
+
 		delimFirstRune, delimFirstRuneLen := utf8.DecodeRuneInString(st.delimiter)
 		delimRuneCount := utf8.RuneCountInString(st.delimiter)
 		switch c {
-		case '\n':
-			// Commands do not require semicolons; newline alone can be delimiter.
-			// Only supported commands so far are USE and DELIMITER.
-			lowerBufStr := strings.ToLower(ls.buf.String())
-			if strings.HasPrefix(lowerBufStr, "use ") || strings.HasPrefix(lowerBufStr, "delimiter ") {
-				ls.doneStatement(0)
-			}
 		case '"', '`', '\'':
 			ls.inQuote = c
 		case delimFirstRune:
@@ -370,7 +395,21 @@ func (ls *lineState) parseStatement() {
 		ls.stmt.Type = StatementTypeNoop
 	} else {
 		sqlStmt := &sqlStatement{}
+		var name *objectName
 		if err := nameParser.ParseString(txt, sqlStmt); err != nil || sqlStmt.forbidden() {
+			if err == nil { // forbidden statement
+				ls.stmt.Type = StatementTypeForbidden
+				ls.stmt.Error = errors.New("Statements such as CREATE TABLE...LIKE and CREATE TABLE...SELECT are not supported")
+			} else if lexErr, ok := err.(*lexer.Error); ok { // lexer error, potentially bad
+				ls.stmt.Type = StatementTypeLexError
+				fileLine, fileCol := ls.stmt.LineNo+lexErr.Tok.Pos.Line-1, lexErr.Tok.Pos.Column
+				if lexErr.Tok.Pos.Line == 1 && ls.stmt.CharNo > 1 { // error is on first line of statement, and statement started mid-line
+					fileCol += ls.stmt.CharNo - 1
+				}
+				ls.stmt.Error = fmt.Errorf("%s:%d:%d: %s", ls.stmt.File, fileLine, fileCol, lexErr.Msg)
+			} else { // unsupported statement, often benign
+				ls.stmt.Error = err // pass through the error as-is (often a participle.UnexpectedTokenError which isn't particularly useful)
+			}
 			return
 		} else if sqlStmt.UseCommand != nil {
 			ls.stmt.Type = StatementTypeCommand
@@ -381,15 +420,19 @@ func (ls *lineState) parseStatement() {
 		} else if sqlStmt.CreateTable != nil {
 			ls.stmt.Type = StatementTypeCreate
 			ls.stmt.ObjectType = tengo.ObjectTypeTable
-			ls.stmt.ObjectQualifier, ls.stmt.ObjectName = sqlStmt.CreateTable.Name.schemaAndTable()
+			name = &sqlStmt.CreateTable.Name
 		} else if sqlStmt.CreateProc != nil {
 			ls.stmt.Type = StatementTypeCreate
 			ls.stmt.ObjectType = tengo.ObjectTypeProc
-			ls.stmt.ObjectQualifier, ls.stmt.ObjectName = sqlStmt.CreateProc.Name.schemaAndTable()
+			name = &sqlStmt.CreateProc.Name
 		} else if sqlStmt.CreateFunc != nil {
 			ls.stmt.Type = StatementTypeCreate
 			ls.stmt.ObjectType = tengo.ObjectTypeFunc
-			ls.stmt.ObjectQualifier, ls.stmt.ObjectName = sqlStmt.CreateFunc.Name.schemaAndTable()
+			name = &sqlStmt.CreateFunc.Name
+		}
+		if name != nil {
+			ls.stmt.ObjectQualifier, ls.stmt.ObjectName = name.schemaAndTable()
+			ls.stmt.nameClause = txt[name.Pos.Offset:name.EndPos.Offset]
 		}
 	}
 }
@@ -472,6 +515,8 @@ func (sqlStmt *sqlStatement) forbidden() bool {
 type objectName struct {
 	Qualifiers []string `parser:"(@Word '.')*"`
 	Name       string   `parser:"@Word"`
+	Pos        lexer.Position
+	EndPos     lexer.Position
 }
 
 // schemaAndTable interprets the ObjectName as a table name which may optionally
