@@ -27,6 +27,23 @@ type Table struct {
 	CreateStatement    string             `json:"showCreateTable"`              // complete SHOW CREATE TABLE obtained from an instance
 }
 
+// ObjectKey returns a value useful for uniquely refering to a Table within a
+// single Schema, for example as a map key.
+func (t *Table) ObjectKey() ObjectKey {
+	if t == nil {
+		return ObjectKey{}
+	}
+	return ObjectKey{
+		Type: ObjectTypeTable,
+		Name: t.Name,
+	}
+}
+
+// Def returns the table's CREATE statement as a string.
+func (t *Table) Def() string {
+	return t.CreateStatement
+}
+
 // AlterStatement returns the prefix to a SQL "ALTER TABLE" statement.
 func (t *Table) AlterStatement() string {
 	return fmt.Sprintf("ALTER TABLE %s", EscapeIdentifier(t.Name))
@@ -156,8 +173,9 @@ func (t *Table) HasAutoIncrement() bool {
 
 // ClusteredIndexKey returns which index is used for an InnoDB table's clustered
 // index. This will be the primary key if one exists; otherwise, it will be the
-// first unique key with non-nullable columns. If there is no such key, or if
-// the table's engine isn't InnoDB, this method returns nil.
+// first unique key made of only non-nullable, non-expression columns. If there
+// is no such key, or if the table's engine isn't InnoDB, this method returns
+// nil.
 func (t *Table) ClusteredIndexKey() *Index {
 	if t.Engine != "InnoDB" {
 		return nil
@@ -168,7 +186,7 @@ func (t *Table) ClusteredIndexKey() *Index {
 	cols := t.ColumnsByName()
 Outer:
 	for _, index := range t.SecondaryIndexes {
-		if index.Unique {
+		if index.Unique && !index.Functional() {
 			for _, part := range index.Parts {
 				if col := cols[part.ColumnName]; col == nil || col.Nullable {
 					continue Outer
